"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.nodeInteractionState = nodeInteractionState;
exports.loop = loop;
exports.initData = initData;
exports.getChangedNodes = getChangedNodes;
exports.loadNodeChild = loadNodeChild;
exports.updateCheckedNodeKeys = updateCheckedNodeKeys;

var _cloneDeep2 = _interopRequireDefault(require("lodash/cloneDeep"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

/**
 *
 * 确定某个节点的选择状态
 * @param {*} { type, node }
 */
function nodeInteractionState() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      _ref$type = _ref.type,
      type = _ref$type === void 0 ? '' : _ref$type,
      _ref$node = _ref.node,
      node = _ref$node === void 0 ? null : _ref$node,
      nodeType = _ref.nodeType,
      _ref$comparateProp = _ref.comparateProp,
      comparateProp = _ref$comparateProp === void 0 ? '' : _ref$comparateProp,
      _ref$attachedComparat = _ref.attachedComparateProp,
      attachedComparateProp = _ref$attachedComparat === void 0 ? '' : _ref$attachedComparat,
      checkedNodes = _ref.checkedNodes,
      intermediateNodeKeys = _ref.intermediateNodeKeys,
      disabledNodeKeys = _ref.disabledNodeKeys;

  if (!type || !node || !comparateProp) {
    throw new Error('nodeInteractionState err');
  }

  var id = node[comparateProp] || '';

  if (type === 'checked') {
    if (_typeof(checkedNodes[0]) === 'object') {
      return checkedNodes.some(function (item) {
        if (item.type !== nodeType) {
          return false;
        }

        if (attachedComparateProp && item[attachedComparateProp] && node[attachedComparateProp]) {
          return "".concat(node[attachedComparateProp]).concat(id) === "".concat(item[attachedComparateProp]).concat(item[comparateProp]);
        }

        return item[comparateProp] === id;
      });
    } else if (typeof checkedNodes[0] === 'string') {
      return checkedNodes.some(function (item) {
        return item === id;
      });
    }
  } else if (type === 'intermediate') {
    return intermediateNodeKeys.some(function (item) {
      return item === id;
    });
  } else if (type === 'disabled') {
    return disabledNodeKeys.some(function (item) {
      return item === id;
    });
  }
}
/**
 *
 * 确定节点的选择状态
 */


function _deptNodeSelectState(_tree) {
  loop({
    list: _tree,
    parentItem: null
  }, function (item) {
    var _children = item._children,
        _isLeaf = item._isLeaf;
    return {
      result: _isLeaf ? null : _children,
      parentItem: item
    };
  }, function (curNode) {
    if (!curNode) {
      return false;
    }

    var _checked = curNode._checked,
        _children = curNode._children;
    var _allUnChecked = true;
    var _allChecked = true;
    var intermediateFlag = false;

    if (_checked) {
      return false;
    }

    _children.forEach(function (childItem) {
      if (childItem._intermediate) {
        intermediateFlag = true;
      }

      if (childItem._checked) {
        _allUnChecked = false;
      } else {
        _allChecked = false;
      }
    });

    curNode._intermediate = intermediateFlag || !(_allUnChecked || _allChecked);
    curNode._checked = _allChecked;
  });
}
/**
 *
 * 深度遍历
 * @param {*} list
 * @param {*} parentItem
 * @param {*} func
 */


function loop(_ref2, func, parentNodeFunc) {
  var list = _ref2.list,
      parentItem = _ref2.parentItem;

  for (var i = 0; i < list.length; i += 1) {
    var item = list[i];
    var next = typeof func === 'function' ? func(item, parentItem) : {
      result: null
    };

    if (next.result !== null) {
      loop({
        list: next.result,
        parentItem: next.parentItem
      }, func, parentNodeFunc);
    }
  }

  if (typeof parentNodeFunc === 'function') {
    parentNodeFunc(parentItem);
  }
}
/**
 * 初始化树结构数据，增加组件辅助属性
 * @param {*} sourceList
 * @param {*} props
 */


function initData(_ref3) {
  var sourceList = _ref3.sourceList,
      props = _ref3.props;
  var staticNodeLabel = props.staticNodeLabel,
      child = props.child,
      staticNodeKey = props.staticNodeKey,
      showCheckbox = props.showCheckbox,
      checkedNodes = props.checkedNodes,
      loadDataFlag = props.loadDataFlag,
      staticNodeTypeLabel = props.staticNodeTypeLabel,
      intermediateNodeKeys = props.intermediateNodeKeys,
      disabledNodeKeys = props.disabledNodeKeys,
      loadData = props.loadData,
      dynamicNodeTypeLabel = props.dynamicNodeTypeLabel,
      dynamicNodeKey = props.dynamicNodeKey,
      dynamicNodeLabel = props.dynamicNodeLabel;
  var tempList = (0, _cloneDeep2.default)(sourceList);
  loop({
    list: tempList,
    parentItem: null
  }, function (item, parentItem) {
    if (item[dynamicNodeLabel]) {
      item._type = dynamicNodeTypeLabel;
      item._id = item[dynamicNodeKey];
      item._children = [];
      item._name = item[dynamicNodeLabel];
      item._isLeaf = true;
    } else {
      item._type = staticNodeTypeLabel;
      item._id = item[staticNodeKey];
      item._children = (0, _cloneDeep2.default)(Array.isArray(item[child]) ? item[child] : []);
      item._name = item[staticNodeLabel];
      item._isLeaf = typeof loadData === 'function' && !item[loadDataFlag] ? false : !item._children.length;
    } // item._type = staticNodeTypeLabel;
    // item._id = item[staticNodeKey];
    // item._name = item[staticNodeLabel];


    item._parentIdList = parentItem && Array.isArray(parentItem._parentIdList) ? parentItem._parentIdList.concat(parentItem._id || '') : [];
    item._isLoaded = typeof loadData === 'function' && !item[loadDataFlag] ? false : null;
    item._allChecked = true;
    item._allUnChecked = true;
    item._disabled = nodeInteractionState({
      type: 'disabled',
      node: item,
      comparateProp: staticNodeKey,
      disabledNodeKeys: disabledNodeKeys
    }) || (parentItem ? parentItem._disabled : false);

    if (showCheckbox) {
      if (parentItem && parentItem._checked) {
        item._checked = true;
        item._intermediate = false;
      } else {
        item._checked = nodeInteractionState({
          type: 'checked',
          node: item,
          nodeType: item._type,
          comparateProp: staticNodeKey,
          checkedNodes: checkedNodes
        });
        item._intermediate = nodeInteractionState({
          type: 'intermediate',
          node: item,
          comparateProp: staticNodeKey,
          intermediateNodeKeys: intermediateNodeKeys
        });
      }
    } else {
      item._checked = null;
      item._intermediate = null;
    } // 删除原有子节点数据，避免数据冗余


    delete item[child];
    return {
      result: item._isLeaf ? null : item._children,
      parentItem: item
    };
  }); // 不获取子节点时，需要对非叶子节点的选择状态进行计算

  if (loadData === null) {
    _deptNodeSelectState(tempList);
  }

  return tempList;
}
/**
 * 补充checkedNodes
 * @param {*} checkedNodesResult
 */


function extendCheckedNodes(_ref4) {
  var _ref4$checkedNodesRes = _ref4.checkedNodesResult,
      checkedNodesResult = _ref4$checkedNodesRes === void 0 ? [] : _ref4$checkedNodesRes,
      checkedNodes = _ref4.checkedNodes,
      dynamicNodeTypeLabel = _ref4.dynamicNodeTypeLabel;
  var result = [];

  if (!Array.isArray(checkedNodes) || !checkedNodes.length || typeof checkedNodes[0] === 'string') {
    return result;
  }

  var users = checkedNodes.filter(function (item) {
    return item.type === dynamicNodeTypeLabel;
  });

  if (!users.length) {
    return result;
  }

  users.forEach(function (item) {
    var _item$userId = item.userId,
        userId = _item$userId === void 0 ? '' : _item$userId,
        _hasloaded = item._hasloaded;

    if (!checkedNodesResult.some(function (node) {
      return node.userId === userId;
    }) && !_hasloaded) {
      result.push(item);
    }
  });
  return result;
}
/**
 * 完成节点选择
 */


function getChangedNodes() {
  var _ref5 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      checkedNodeProps = _ref5.checkedNodeProps,
      _tree = _ref5._tree,
      _ref5$filterFunc = _ref5.filterFunc,
      filterFunc = _ref5$filterFunc === void 0 ? null : _ref5$filterFunc,
      _ref5$loopStyle = _ref5.loopStyle,
      loopStyle = _ref5$loopStyle === void 0 ? 'default' : _ref5$loopStyle,
      uncheckNode = _ref5.uncheckNode,
      callbackUncheck = _ref5.callbackUncheck;

  var checkedNodesResult = [];
  var intermediateNodes = [];

  if (!checkedNodeProps.length) {
    throw new Error('checkedNodeProps 长度不可为 0');
  } // 取消选中项


  if (uncheckNode && Object.keys(uncheckNode).length) {
    var handleTest = function handleTest(data, detail, isok) {
      data.map(function (item) {
        if (isok) {
          item._allChecked = !1;
          item._checked = !1;
          item._allUnChecked = !1;

          if (item._children && item._children.length) {
            handleTest(item._children, detail, isok);
          }

          return true;
        }

        if (detail.deptName === item.deptName && item._type === detail.type) {
          if (item._type === 'user' && item.userId === detail.userId) {
            item._intermediate = !1;
            item._checked = !1;
            return true;
          } else if (item._type === 'dept' && detail.deptId === item.deptId) {
            item._allChecked = !1;
            item._checked = !1; // eslint-disable-next-line no-unused-expressions

            item._children && item._children.length && handleTest(item._children, detail, !0);
          }
        }

        if (item._children && item._children.length) {
          handleTest(item._children, detail);
        }

        return true;
      });
    };

    handleTest(_tree, uncheckNode);

    if (typeof callbackUncheck === 'function') {
      callbackUncheck();
    }
  }

  loop({
    list: _tree
  }, function (item) {
    var _type = item._type;
    var tempItem = {
      type: _type
    };
    checkedNodeProps.forEach(function (key) {
      tempItem[key] = item[key] === undefined ? '' : item[key];
    });
    var _checked = item._checked,
        _intermediate = item._intermediate,
        _isLeaf = item._isLeaf,
        _children = item._children;

    if (_checked) {
      if (typeof filterFunc === 'function') {
        if (filterFunc((0, _cloneDeep2.default)(tempItem))) {
          checkedNodesResult.push(tempItem);
        }
      } else {
        checkedNodesResult.push(tempItem);
      }

      if (loopStyle === 'default') {
        return {
          result: null
        };
      }
    }

    if (_intermediate) {
      intermediateNodes.push(tempItem);
    }

    return {
      result: _isLeaf ? null : _children
    };
  });
  return {
    // checkedNodes: checkedNodesResult.concat(
    //   extendCheckedNodes({
    //     intermediateNodes,
    //     checkedNodesResult,
    //     checkedNodes,
    //     dynamicNodeTypeLabel,
    //   })
    // ),
    checkedNodes: checkedNodesResult,
    intermediateNodes: intermediateNodes
  };
}
/**
 *
 * 挂载子节点
 * @param {*} node
 * @param {*} childList
 * @param {*} treeProps
 */


function loadNodeChild() {
  var _ref6 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      _ref6$node = _ref6.node,
      node = _ref6$node === void 0 ? null : _ref6$node,
      _ref6$childList = _ref6.childList,
      childList = _ref6$childList === void 0 ? [] : _ref6$childList,
      _ref6$treeProps = _ref6.treeProps,
      treeProps = _ref6$treeProps === void 0 ? {} : _ref6$treeProps,
      checkedNodes = _ref6.checkedNodes,
      disabledNodeKeys = _ref6.disabledNodeKeys;

  if (!node) {
    throw new Error('param node is need.');
  }

  var dynamicNodeLabel = treeProps.dynamicNodeLabel,
      dynamicNodeKey = treeProps.dynamicNodeKey,
      staticNodeKey = treeProps.staticNodeKey,
      dynamicNodeTypeLabel = treeProps.dynamicNodeTypeLabel;
  var parentNodeId = node._id,
      parentNodeChecked = node._checked,
      parentNodeDisabled = node._disabled;

  var filterChildren = node._children.filter(function (item) {
    return item.type !== 'user';
  });

  node._children = filterChildren;
  childList.forEach(function (item) {
    node._children.push(_objectSpread({}, item, {
      _id: item[dynamicNodeKey],
      _name: item[dynamicNodeLabel],
      _children: [],
      _isLeaf: true,
      _checked: parentNodeChecked || nodeInteractionState({
        type: 'checked',
        node: _objectSpread({}, item),
        nodeType: dynamicNodeTypeLabel,
        comparateProp: dynamicNodeKey,
        attachedComparateProp: staticNodeKey,
        checkedNodes: checkedNodes
      }),
      _intermediate: false,
      _type: dynamicNodeTypeLabel,
      _disabled: nodeInteractionState({
        type: 'disabled',
        node: _objectSpread({}, item),
        comparateProp: dynamicNodeKey,
        disabledNodeKeys: disabledNodeKeys
      }) || parentNodeDisabled
    }));
  }); // 更新节点状态

  node._isLoaded = true;
  node._isLeaf = !node._children.length;
  updateCheckedNodeKeys({
    deptId: parentNodeId,
    dynamicNodeTypeLabel: dynamicNodeTypeLabel,
    checkedNodes: checkedNodes
  });
}
/**
 * 更新本地缓存的_checkedNodeKeys，解决getChangedNodes错误添加的问题
 * @param {*} deptId 某个父节点的id
 */


function updateCheckedNodeKeys() {
  var _ref7 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      _ref7$deptId = _ref7.deptId,
      deptId = _ref7$deptId === void 0 ? '' : _ref7$deptId,
      dynamicNodeTypeLabel = _ref7.dynamicNodeTypeLabel,
      checkedNodes = _ref7.checkedNodes;

  if (!Array.isArray(checkedNodes) || !checkedNodes.length || typeof checkedNodes[0] === 'string') {
    return false;
  }

  checkedNodes.forEach(function (item) {
    var type = item.type,
        _item$deptId = item.deptId,
        _deptId = _item$deptId === void 0 ? '' : _item$deptId;

    if (type === dynamicNodeTypeLabel && _deptId === deptId) {
      item._hasloaded = true;
    }
  });
}