"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.rollNodeToWhere = rollNodeToWhere;
exports.getNodesByKey = getNodesByKey;
exports.loadUserPromise = loadUserPromise;

var _tree2 = require("./tree");

var _tween = _interopRequireDefault(require("./tween"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// 是否正在执行滚动动画
var _scrollAnimate = false;
/**
 *
 * 缓动DOMElement的某个属性值
 * @param {*} { ele, prop, intVal }
 */

function animateChange(_ref) {
  var ele = _ref.ele,
      currentVal = _ref.currentVal,
      changeVal = _ref.changeVal,
      prop = _ref.prop;

  /* eslint-disable no-param-reassign */
  var duration = 500;
  var startTime = +new Date();

  (function stepAnimate() {
    var t = +new Date();

    if (t >= startTime + duration) {
      ele[prop] = currentVal + changeVal;
      _scrollAnimate = false;
      return false;
    }

    var _curPropVal = _tween.default.Expo.easeOut(t - startTime, currentVal, changeVal, duration);

    ele[prop] = _curPropVal;
    requestAnimationFrame(stepAnimate);
  })();
  /* eslint-enable */

}
/**
 *
 * 计算需要滚动的距离
 */


function getScrollEndDis(_ref2) {
  var ele = _ref2.ele,
      wrapEle = _ref2.wrapEle,
      nodeWrapEle = _ref2.nodeWrapEle;
  var offsetTop = ele.offsetTop;
  var wrapHeight = wrapEle.clientHeight;
  var contentHeight = nodeWrapEle.clientHeight; // 可滚动的最大距离

  var scrollMaxDis = contentHeight - wrapHeight; // 滚动结束位置

  var scrollEndDis = 0;

  if (offsetTop <= wrapHeight * 0.5) {
    scrollEndDis = 0;
  } else {
    scrollEndDis = offsetTop - wrapHeight + wrapHeight * 0.5;
  }

  scrollEndDis = scrollEndDis > scrollMaxDis ? scrollMaxDis : scrollEndDis;
  return scrollEndDis;
}
/**
 *
 * 将指定节点滚动至可视区域中央（或底部）
 * @param {*} { ele, wrapEle }
 */


function rollNodeToWhere() {
  var _ref3 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      _ref3$ele = _ref3.ele,
      ele = _ref3$ele === void 0 ? null : _ref3$ele,
      _ref3$wrapEle = _ref3.wrapEle,
      wrapEle = _ref3$wrapEle === void 0 ? null : _ref3$wrapEle;

  /* eslint-disable */
  if (!ele || !wrapEle || _scrollAnimate) {
    return;
  }

  _scrollAnimate = true;
  var nodeWrapEle = document.getElementsByClassName('nbugs-component-tree-node')[0];
  var scrollStartDis = wrapEle.scrollTop;
  var scrollEndDis = getScrollEndDis({
    ele: ele,
    wrapEle: wrapEle,
    nodeWrapEle: nodeWrapEle
  });
  var changeVal = scrollEndDis - scrollStartDis;

  if (changeVal === 0) {
    return false;
  }

  animateChange({
    ele: wrapEle,
    currentVal: scrollStartDis,
    changeVal: changeVal,
    prop: 'scrollTop'
  });
  /* eslint-enable */
}
/**
 *
 * 模糊搜索树节点，并对结果加以限制
 * key: 部门名称关键字或者部门id
 * total: 'infinity'/Number
 * @param {*} { key, total }
 */


function getNodesByKey() {
  var _ref4 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      _ref4$_tree = _ref4._tree,
      _tree = _ref4$_tree === void 0 ? [] : _ref4$_tree,
      _ref4$key = _ref4.key,
      key = _ref4$key === void 0 ? '' : _ref4$key,
      _ref4$staticNodeTypeL = _ref4.staticNodeTypeLabel,
      staticNodeTypeLabel = _ref4$staticNodeTypeL === void 0 ? '' : _ref4$staticNodeTypeL,
      _ref4$total = _ref4.total,
      total = _ref4$total === void 0 ? 10 : _ref4$total;

  if (!Array.isArray(_tree) || !_tree.length) {
    throw new Error('树结构未加载完成！');
  }

  if (!key) {
    return [];
  }

  var tempList = [];
  (0, _tree2.loop)({
    list: _tree,
    parentItem: null
  }, function (item) {
    var _name = item._name,
        _id = item._id,
        _parentIdList = item._parentIdList,
        _type = item._type;

    if (_type === staticNodeTypeLabel && (_name.indexOf(key) !== -1 || _id === key)) {
      tempList.push({
        deptName: _name,
        deptId: _id,
        parentIdList: _parentIdList
      });
    }

    return {
      result: item._isLeaf ? null : item._children
    };
  });
  var resultList = total === 'infinity' ? tempList : tempList.slice(0, total);
  return resultList;
}
/**
 * loadData 过程中产生的 promise 管理
 * 方便 Tree 组件管理受控属性更新时机
 */


var promises = [];

function loadUserPromise() {
  return {
    getPromises: function getPromises() {
      return promises;
    },
    clearPromises: function clearPromises() {
      promises.length = 0;
    },
    addPromises: function addPromises(loadPromise) {
      promises.push(loadPromise);
    }
  };
}