import React, { Component } from 'react';
import classNames from 'classnames';
import { connect } from 'dva-no-router';
import './Checkbox.less';
import checkCircleFill from '../../assets/image/check-circle-fill.svg';
import checkSquareFill from '../../assets/image/check-square-fill.svg';
import loadingIcon from '../../assets/image/loading.svg';
import checkDisabled from '../../assets/image/check_disabled.svg';

class Checkbox extends Component {
  constructor(props) {
    super(props);

    this.renderContent = this.renderContent.bind(this);
  }

  renderDisabled() {
    const { className } = this.props;

    const checkboxCls = classNames(['nbugs-component-checkbox', className]);

    return (
      <div className={checkboxCls}>
        <img className="check-box-checked" src={checkDisabled} alt="disabled" />
      </div>
    );
  }

  renderContent() {
    const { className, checked, intermediate, disabled, loading, styleType, themeImg } = this.props;
    const checkboxCls = classNames(['nbugs-component-checkbox', className]);

    if (loading) {
      return (
        <div className={checkboxCls}>
          {/* 节点加载状态 */}
          {loading && (
            <img className={classNames('check-box-loading')} src={loadingIcon} alt="loading" />
          )}
        </div>
      );
    }

    // 禁用状态
    if (disabled) {
      return this.renderDisabled();
    }

    let imgSrc = styleType === 'mobile' ? checkCircleFill : checkSquareFill;

    if (themeImg) {
      imgSrc = themeImg;
    }

    return (
      <div className={checkboxCls}>
        {/* 默认状态 */}
        {styleType === 'mobile' &&
          (!checked && !intermediate && <div className={classNames('check-box-default')} />)}
        {styleType === 'pc' &&
          (!checked && !intermediate && <div className={classNames('check-box-default-pc')} />)}
        {/* 选中状态 */}
        {checked && (
          <img
            className={
              styleType === 'mobile'
                ? classNames('check-box-checked')
                : classNames('check-box-checked-pc')
            }
            src={imgSrc}
            alt="checked"
          />
        )}
        {/* 过渡状态 */}
        {!checked && intermediate && (
          <img
            className={
              styleType === 'mobile'
                ? classNames('check-box-intermediate')
                : classNames('check-box-intermediate-pc')
            }
            src={imgSrc}
            alt="intermediate"
          />
        )}
      </div>
    );
  }

  render() {
    return this.renderContent();
  }
}

export default connect()(Checkbox);
