/* eslint-disable no-console, no-param-reassign, no-shadow  */
import React from 'react';
import ReactDOM from 'react-dom';
import { getAddress, getDepartUser } from './service/tree';
import { Tree, getNodesByKey } from './component.js';

const corpid = 'wx981d0202130ed8e7';
// const checkedNodes = [
//   {
//     deptId: '086A8D4C1D1A4CCCBAD59A83E63F6B25',
//     deptName: '教职工组',
//     deptType: 'schoolworker_group',
//     type: 'user',
//     userId: '06A229E7D13D400EAA54FDDDCA4EBE52',
//     userName: '周信',
//     userType: '',
//   },
//   {
//     deptId: '3C4E78BE4ACF47B3B677715E3CB46B46',
//     deptName: '学生组',
//     deptType: 'student_group',
//     memberRole: '',
//     type: 'dept',
//   },
// ];
const checkedNodes = [];
// const intermediateNodeKeys = [
//   '330108-S000018',
//   '381BBDDDD6AC43198CF01C88FE8BC17D',
//   '086A8D4C1D1A4CCCBAD59A83E63F6B25',
// ];
const intermediateNodeKeys = [];
const disabledNodeKeys = [];
// const expandedNodeKeys = [];

class Test extends React.Component {
  static fetchDepartUser(params) {
    const { deptId, deptType, deptName } = params;

    return getDepartUser({
      corpid,
      deptId,
      deptType,
    }).then(data => {
      const { users = [] } = data;
      return users.map(item => {
        const { userId, userName, userType = '' } = item;
        return {
          userId,
          userName,
          userType,
          deptId,
          deptName,
          deptType,
        };
      });
    });
  }

  static filterNodes(originNodes) {
    const _nodes = originNodes.map(item => {
      const { type } = item;

      if (type === 'dept') {
        delete item.userId;
        delete item.userName;
        delete item.userType;
      } else if (type === 'user') {
        delete item.memberRole;
        delete item.deptType;
      }

      return item;
    });
    return _nodes;
  }

  static beforeNodeChange({ node, checkedNodes }) {
    console.log('TCL: Test -> staticbeforeNodeChange -> checkedNodes', checkedNodes);
    console.log('TCL: Test -> staticbeforeNodeChange -> node', node);
    return true;
  }

  static handleConfirm({ checkedNodes = [], intermediateNodes = [] } = {}) {
    const _checkedList = Test.filterNodes(checkedNodes);
    const deptList = _checkedList.filter(item => item.type === 'dept');
    const userList = _checkedList.filter(item => item.type === 'user');
    Test.filterNodes(intermediateNodes);
    console.log('​Test -> statichandleConfirm -> deptList', deptList);
    console.log('​Test -> statichandleConfirm -> userList', userList);
    console.log('​Test -> statichandleConfirm -> intermediateNodes', intermediateNodes);
    console.log('​Test -> statichandleConfirm -> checkedNodes', checkedNodes);
  }

  static handleCancel() {
    console.log('点击了取消按钮');
  }

  static filterFunc(node) {
    const { deptType } = node;
    return deptType === 'general_class';
  }

  static searchNodeByKey() {
    // 7B300B2163874130899B76BF5ECB58ED
    console.log(getNodesByKey({ key: '总务处' }));
  }

  constructor(props) {
    super(props);

    this.state = {
      sourceList: [],
      expandedNodeKeys: [],
      highlightId: '',
      dynamicCheckedId: '',
    };
  }

  componentWillMount() {
    this.fetchTree();
  }

  fetchTree() {
    getAddress({ corpid }).then(data => {
      const treeData = data.rootDepartments;
      this.setState({
        sourceList: treeData,
      });
    });
  }

  selectDept() {
    this.setState({
      expandedNodeKeys: [
        '330108-S000018',
        '381BBDDDD6AC43198CF01C88FE8BC17D',
        '086A8D4C1D1A4CCCBAD59A83E63F6B25',
      ],
      dynamicCheckedId: '77A1E49F4CA54ED9A52792A048228B92',
    });
  }

  selectUser() {
    this.setState({
      expandedNodeKeys: [
        '330108-S000018',
        'AF5F4EF1ED934BB8A4232B035FDECA82',
        '081D4F6EC46F4328AA495F0C0BEE6838',
      ],
      dynamicCheckedId: '65DC2DBD9FB34903AC405605FEE7EB1A',
    });
  }

  render() {
    const checkedNodeProps = [
      'deptId',
      'deptName',
      'memberRole',
      'userId',
      'userName',
      'userType',
      'deptType',
    ];
    const nodeFoldDetailProps = ['deptId', 'deptName', 'deptType'];
    const { sourceList, expandedNodeKeys, highlightId, dynamicCheckedId } = this.state;

    if (!sourceList.length) {
      return null;
    }

    return (
      <div style={{ height: '100%' }}>
        <Tree
          treeData={sourceList}
          staticNodeLabel="deptName"
          dynamicNodeLabel="userName"
          staticNodeKey="deptId"
          dynamicNodeKey="userId"
          child="subDepts"
          platform="pc" // pc/mobile
          // deptNodeCheckable={false}
          checkedNodeProps={checkedNodeProps}
          checkedNodes={checkedNodes}
          intermediateNodeKeys={intermediateNodeKeys}
          expandedNodeKeys={expandedNodeKeys}
          disabledNodeKeys={disabledNodeKeys}
          highlightId={highlightId}
          dynamicCheckedId={dynamicCheckedId}
          loadData={Test.fetchDepartUser}
          // loadData={null}
          nodeFoldDetailProps={nodeFoldDetailProps}
          // beforeNodeChange={Test.beforeNodeChange}
          // filterFunc={Test.filterFunc}
          // loopStyle="entire"
          onCancel={Test.handleCancel}
          onOk={Test.handleConfirm}
        />
        <p style={{ margin: '30px' }} onClick={this.selectDept.bind(this)}>
          选中部门
        </p>
        <p style={{ margin: '30px' }} onClick={this.selectUser.bind(this)}>
          选中人员
        </p>
        <p style={{ margin: '30px' }} onClick={Test.searchNodeByKey.bind(this)}>
          搜索部门节点
        </p>
      </div>
    );
  }
}

ReactDOM.render(<Test />, document.getElementById('root'));
