import React, { Component } from 'react';
import classNames from 'classnames';
import './Tree.less';
import { connect } from 'dva-no-router';
import { getChangedNodes } from '../../utils/tree';
import { loadUserPromise } from '../../utils/treePC';
import TreeNode from '../../components/TreeNode';

const { getPromises, clearPromises } = loadUserPromise();

class App extends Component {
  constructor(props) {
    super(props);

    this.state = {
      okBtnDisabled: true,
    };

    this.renderContent = this.renderContent.bind(this);
    this.handleCancel = this.handleCancel.bind(this);
    this.handleConfirm = this.handleConfirm.bind(this);
    this.resetHighlightId = this.resetHighlightId.bind(this);
    this.handleNodesChange = this.handleNodesChange.bind(this);
  }

  componentWillMount() {
    const { componentProps } = this.props;
    const { checkedNodes } = componentProps;

    this.setState({
      okBtnDisabled: !checkedNodes.length,
    });
  }

  // Tree更新完成之后，reset 受控属性
  componentDidUpdate() {
    const { componentProps } = this.props;
    const promises = getPromises();

    if (Array.isArray(promises) && promises.length) {
      Promise.all(promises).finally(() => {
        componentProps.dynamicCheckedId = '';
        componentProps.dynamicUnCheckedId = '';
        componentProps.expandedNodeKeys = [];
        clearPromises();
      });
    } else {
      componentProps.dynamicCheckedId = '';
      componentProps.dynamicUnCheckedId = '';
      componentProps.expandedNodeKeys = [];
    }
  }

  // 取消
  handleCancel() {
    const { componentProps } = this.props;
    const { onCancel } = componentProps;

    if (typeof onCancel === 'function') {
      onCancel();
    }
  }

  // 完成人员选择
  handleConfirm() {
    const { okBtnDisabled } = this.state;
    const { componentProps, _tree } = this.props;
    const {
      onOk,
      filterFunc,
      loopStyle,
      dynamicNodeTypeLabel,
      checkedNodeProps = [],
    } = componentProps;

    if (okBtnDisabled) {
      return null;
    }

    if (typeof onOk === 'function') {
      const { checkedNodes, intermediateNodes } = getChangedNodes({
        checkedNodeProps,
        filterFunc,
        loopStyle,
        _tree,
        checkedNodes: componentProps.checkedNodes,
        dynamicNodeTypeLabel,
      });
      onOk({ checkedNodes, intermediateNodes });
    }
  }

  resetHighlightId() {
    const { dispatch } = this.props;
    dispatch({
      type: 'tree/resetHighlightId',
    });
  }

  handleNodesChange({ checkedNodes }) {
    this.setState({
      okBtnDisabled: !checkedNodes.length,
    });
  }

  renderContent() {
    const { okBtnDisabled } = this.state;
    const { componentProps, _tree } = this.props;
    const { className, showOptionsBtn, themeColor } = componentProps;
    const treeCls = classNames(['nbugs-component-tree', className]);

    if (!_tree.length) {
      return null;
    }

    return (
      <div className={treeCls}>
        <div className={classNames('tree-wrap')}>
          {_tree.map(item => {
            return (
              <TreeNode
                key={item._id}
                treeNode={item}
                treeProps={componentProps}
                resetHighlightId={this.resetHighlightId}
                onNodesChange={this.handleNodesChange}
              />
            );
          })}
        </div>
        {/* 操作按钮组 */}
        {showOptionsBtn && (
          <div className={classNames('btn-group')}>
            {/* 确定按钮 */}
            <div className={classNames('btn-cancel')} onClick={this.handleCancel}>
              取消
            </div>
            {/* 确定按钮 */}
            <div
              className={
                okBtnDisabled ? classNames(['btn-ok', 'btn-disabled']) : classNames('btn-ok')
              }
              style={{
                background: themeColor,
              }}
              onClick={this.handleConfirm}
            >
              确定
            </div>
          </div>
        )}
      </div>
    );
  }

  render() {
    return this.renderContent();
  }
}

export default connect(({ global, ...rest }) => ({
  ...global,
  ...rest.tree,
}))(App);
