import React, { Component } from 'react';
import PropTypes from 'prop-types';
import Tree from './Tree';
import createDva from '../../dva/dva';
import { getNodesByKey, loadUserPromise } from '../../utils/treePC';
import { getChangedNodes } from '../../utils/tree';

export default class TreeWrap extends Component {
  /* eslint-disable react/require-default-props, react/no-unused-prop-types */
  static propTypes = {
    treeData: PropTypes.arrayOf(PropTypes.object).isRequired,
    child: PropTypes.string.isRequired,
    platform: PropTypes.oneOf(['pc', 'mobile']).isRequired,
    staticNodeLabel: PropTypes.string.isRequired,
    dynamicNodeLabel: PropTypes.string.isRequired,
    staticNodeKey: PropTypes.string.isRequired,
    dynamicNodeKey: PropTypes.string.isRequired,
    checkedNodeProps: PropTypes.arrayOf(PropTypes.string).isRequired,
    nodeFoldDetailProps: PropTypes.array,
    staticNodeTypeLabel: PropTypes.string,
    dynamicNodeTypeLabel: PropTypes.string,
    loadData: PropTypes.func,
    loadDataFlag: PropTypes.string,
    filterFunc: PropTypes.func,
    loopStyle: PropTypes.oneOf(['default', 'entire']),
    showOptionsBtn: PropTypes.bool,
    showCheckbox: PropTypes.bool,
    expandRootNode: PropTypes.bool,
    deptNodeCheckable: PropTypes.bool,
    checkedNodes: PropTypes.array,
    intermediateNodeKeys: PropTypes.array,
    expandedNodeKeys: PropTypes.array,
    disabledNodeKeys: PropTypes.array,
    dynamicCheckedId: PropTypes.string,
    dynamicUnCheckedId: PropTypes.string,
    onOk: PropTypes.func,
    onCancel: PropTypes.func,
    onChange: PropTypes.func,
    className: PropTypes.string,
    themeColor: PropTypes.string,
    themeImg: PropTypes.object,
    renderNodeText: PropTypes.func,
  };
  /* eslint-enable */

  constructor(props) {
    super(props);

    this.dvaInstance = createDva(<Tree />);

    this.handleUpdateState(props, true);
  }

  componentWillReceiveProps(nextProps) {
    this.handleUpdateState(nextProps, false);
  }

  // 根据节点名称或者是节点id搜索
  getNodesByKey = ({ key = '', total = 10 } = {}) => {
    const { app } = this.dvaInstance;
    const treeState = app._store.getState().tree;
    const { _tree, componentProps } = treeState;
    const { staticNodeTypeLabel } = componentProps;
    return getNodesByKey({
      _tree,
      key,
      total,
      staticNodeTypeLabel,
    });
  };

  // 获取变化的节点列表 增加参数（uncheckNode）取消选中
  getChangedNodes = () => {
    const { app } = this.dvaInstance;
    const treeState = app._store.getState().tree;
    const { _tree, componentProps } = treeState;
    const {
      checkedNodeProps,
      dynamicNodeTypeLabel,
      checkedNodes,
      filterFunc,
      loopStyle,
      uncheckNode,
      callbackUncheck,
    } = componentProps;

    return getChangedNodes({
      checkedNodeProps,
      dynamicNodeTypeLabel,
      _tree,
      filterFunc,
      loopStyle,
      checkedNodes,
      uncheckNode,
      callbackUncheck,
    });
  };

  // 获取 loadData 过程中产生的 promise 管理权限
  loadUserPromise = () => {
    return loadUserPromise();
  };

  // 初始化数据
  handleUpdateState = (props, init) => {
    const { app } = this.dvaInstance;

    app._store.dispatch({
      type: 'tree/initialData',
      payload: {
        props,
        init,
      },
    });

    // 首次不执行 && uncheckNode有数据才调用
    if (!init && props.uncheckNode) {
      this.getChangedNodes()
    }
  };

  render() {
    const { Provider, app } = this.dvaInstance;
    return (
      <Provider store={app._store}>
        <Tree />
      </Provider>
    );
  }
}
