import { isEqual } from 'lodash';
import { initData } from '../../utils/tree';

const initState = {
  componentProps: {
    checkedNodes: [],
    intermediateNodeKeys: [],
    expandedNodeKeys: [],
    disabledNodeKeys: [],
    dynamicCheckedId: '',
    dynamicUnCheckedId: '',
    nodeFoldDetailProps: [],
    deptNodeCheckable: true,
    showCheckbox: true,
    showOptionsBtn: true,
    expandRootNode: true,
    onOk: null,
    onCancel: null,
    onChange: null,
    filterFunc: null,
    loopStyle: 'default',
    loadData: null,
    loadDataFlag: '',
    staticNodeTypeLabel: 'dept',
    dynamicNodeTypeLabel: 'user',
    className: '',
    themeColor: '#1786EC',
    themeImg: null,
  },
  updateCheckedNodes: false, // 告知TreeNode是否需要care checkedNodes
  updateIntermediateNodeKeys: false, // 告知TreeNode是否需要care checkedNodes
  _tree: [], // 树的本地缓存数据
};

export default {
  namespace: 'tree',

  state: initState,

  effects: {},

  reducers: {
    resetState() {
      return initState;
    },
    saveState(state, { payload }) {
      return {
        ...state,
        ...payload,
      };
    },
    // 初始化数据
    initialData(state, { payload }) {
      const { props, init } = payload;
      const { treeData, checkedNodes } = props;
      const componentProps = Object.assign({}, state.componentProps, props);
      const updateCheckedNodes = !isEqual(state.componentProps.checkedNodes, props.checkedNodes);
      const updateIntermediateNodeKeys = !isEqual(
        state.componentProps.intermediateNodeKeys,
        props.intermediateNodeKeys
      );

      function treeItem(obj, checkData) {
        const _obj = Object.assign({}, obj);
        // eslint-disable-next-line guard-for-in
        for (const i in _obj) {
          if (checkData[props.staticNodeKey] === _obj[i][props.staticNodeKey]) {
            if (_obj[i][props.child]) {
              _obj[i][props.child].push(checkData);
            }
          }
          if (_obj[i][props.child]) {
            treeItem(_obj[i][props.child], checkData);
          }
        }
      }

      for (const i in checkedNodes) {
        if (checkedNodes[i].type === 'user') {
          treeItem(props.treeData, checkedNodes[i]);
        }
      }

      if (init) {
        return {
          ...state,
          componentProps,
          _tree: initData({ sourceList: treeData, props: componentProps }),
        };
      } else {
        return {
          ...state,
          componentProps,
          updateCheckedNodes,
          updateIntermediateNodeKeys,
        };
      }
    },
  },
};
