import { loop } from './tree';
import Tween from './tween';

// 是否正在执行滚动动画
let _scrollAnimate = false;

/**
 *
 * 缓动DOMElement的某个属性值
 * @param {*} { ele, prop, intVal }
 */
function animateChange({ ele, currentVal, changeVal, prop }) {
  /* eslint-disable no-param-reassign */
  const duration = 500;
  const startTime = +new Date();

  (function stepAnimate() {
    const t = +new Date();
    if (t >= startTime + duration) {
      ele[prop] = currentVal + changeVal;
      _scrollAnimate = false;
      return false;
    }

    const _curPropVal = Tween.Expo.easeOut(t - startTime, currentVal, changeVal, duration);
    ele[prop] = _curPropVal;
    requestAnimationFrame(stepAnimate);
  })();
  /* eslint-enable */
}

/**
 *
 * 计算需要滚动的距离
 */
function getScrollEndDis({ ele, wrapEle, nodeWrapEle }) {
  const { offsetTop } = ele;
  const wrapHeight = wrapEle.clientHeight;
  const contentHeight = nodeWrapEle.clientHeight;
  // 可滚动的最大距离
  const scrollMaxDis = contentHeight - wrapHeight;
  // 滚动结束位置
  let scrollEndDis = 0;
  if (offsetTop <= wrapHeight * 0.5) {
    scrollEndDis = 0;
  } else {
    scrollEndDis = offsetTop - wrapHeight + wrapHeight * 0.5;
  }
  scrollEndDis = scrollEndDis > scrollMaxDis ? scrollMaxDis : scrollEndDis;

  return scrollEndDis;
}

/**
 *
 * 将指定节点滚动至可视区域中央（或底部）
 * @param {*} { ele, wrapEle }
 */
export function rollNodeToWhere({ ele = null, wrapEle = null } = {}) {
  /* eslint-disable */
  if (!ele || !wrapEle || _scrollAnimate) {
    return;
  }

  _scrollAnimate = true;

  const nodeWrapEle = document.getElementsByClassName('nbugs-component-tree-node')[0];
  const scrollStartDis = wrapEle.scrollTop;
  const scrollEndDis = getScrollEndDis({ ele, wrapEle, nodeWrapEle });
  const changeVal = scrollEndDis - scrollStartDis;

  if (changeVal === 0) {
    return false;
  }

  animateChange({
    ele: wrapEle,
    currentVal: scrollStartDis,
    changeVal,
    prop: 'scrollTop',
  });
  /* eslint-enable */
}

/**
 *
 * 模糊搜索树节点，并对结果加以限制
 * key: 部门名称关键字或者部门id
 * total: 'infinity'/Number
 * @param {*} { key, total }
 */
export function getNodesByKey({ _tree = [], key = '', staticNodeTypeLabel = '', total = 10 } = {}) {
  if (!Array.isArray(_tree) || !_tree.length) {
    throw new Error('树结构未加载完成！');
  }

  if (!key) {
    return [];
  }

  const tempList = [];

  loop({ list: _tree, parentItem: null }, item => {
    const { _name, _id, _parentIdList, _type } = item;
    if (_type === staticNodeTypeLabel && (_name.indexOf(key) !== -1 || _id === key)) {
      tempList.push({
        deptName: _name,
        deptId: _id,
        parentIdList: _parentIdList,
      });
    }
    return {
      result: item._isLeaf ? null : item._children,
    };
  });

  const resultList = total === 'infinity' ? tempList : tempList.slice(0, total);

  return resultList;
}

/**
 * loadData 过程中产生的 promise 管理
 * 方便 Tree 组件管理受控属性更新时机
 */
const promises = [];
export function loadUserPromise() {
  return {
    getPromises() {
      return promises;
    },
    clearPromises() {
      promises.length = 0;
    },
    addPromises(loadPromise) {
      promises.push(loadPromise);
    },
  };
}
